@@ -38,6 +38,7 @@ struct kanshi_head {
 	int32_t x, y;
 	enum wl_output_transform transform;
 	double scale;
+	bool adaptive_sync;
 };
 
 struct kanshi_state {
@@ -64,6 +65,7 @@ struct kanshi_pending_profile {
 };
 
 bool kanshi_reload_config(struct kanshi_state *state);
+bool kanshi_switch(struct kanshi_state *state, struct kanshi_profile *profile);
 
 int kanshi_main_loop(struct kanshi_state *state);
 
