@@ -20,7 +20,7 @@
 
 #define HEADS_MAX 64
 
-static bool try_apply_profiles(struct kanshi_state *state);
+static bool match_and_apply(struct kanshi_state *state);
 
 static bool match_profile_output(struct kanshi_profile_output *output,
 		struct kanshi_head *head) {
@@ -131,25 +131,24 @@ static void exec_command(char *cmd) {
 	}
 }
 
-static void execute_profile_commands(struct kanshi_profile *profile) {
-	struct kanshi_profile_command *command;
-	wl_list_for_each(command, &profile->commands, link) {
-		fprintf(stderr, "Running command '%s'\n", command->command);
-		exec_command(command->command);
-	}
-}
-
 static void config_handle_succeeded(void *data,
 		struct zwlr_output_configuration_v1 *config) {
 	struct kanshi_pending_profile *pending = data;
 	zwlr_output_configuration_v1_destroy(config);
-	fprintf(stderr, "running commands for configuration '%s'\n", pending->profile->name);
-	execute_profile_commands(pending->profile);
-	fprintf(stderr, "configuration for profile '%s' applied\n",
-			pending->profile->name);
-	pending->state->current_profile = pending->profile;
-	if (pending->profile == pending->state->pending_profile) {
-		pending->state->pending_profile = NULL;
+
+	struct kanshi_state *state = pending->state;
+	struct kanshi_profile *profile = pending->profile;
+
+	struct kanshi_profile_command *command;
+	wl_list_for_each(command, &profile->commands, link) {
+		fprintf(stderr, "running command '%s'\n", command->command);
+		exec_command(command->command);
+	}
+
+	fprintf(stderr, "configuration for profile '%s' applied\n", profile->name);
+	state->current_profile = profile;
+	if (profile == state->pending_profile) {
+		state->pending_profile = NULL;
 	}
 	free(pending);
 }
@@ -179,7 +178,7 @@ static void config_handle_cancelled(void *data,
 	if (pending->serial != pending->state->serial) {
 		// We've already received a new serial, try re-applying the profile
 		// immediately
-		try_apply_profiles(pending->state);
+		match_and_apply(pending->state);
 	}
 	free(pending);
 }
@@ -300,6 +299,10 @@ static void apply_profile(struct kanshi_state *state,
 			zwlr_output_configuration_head_v1_set_transform(config_head,
 				profile_output->transform);
 		}
+		if (profile_output->fields & KANSHI_OUTPUT_ADAPTIVE_SYNC) {
+			zwlr_output_configuration_head_v1_set_adaptive_sync(config_head,
+				profile_output->adaptive_sync);
+		}
 	}
 
 	zwlr_output_configuration_v1_apply(config);
@@ -461,6 +464,12 @@ void head_handle_serial_number(void *data,
 	head->serial_number = strdup(serial_number);
 }
 
+static void head_handle_adaptive_sync(void *data,
+		struct zwlr_output_head_v1 *zwlr_output_head_v1, uint32_t state) {
+	struct kanshi_head *head = data;
+	head->adaptive_sync = state;
+}
+
 static const struct zwlr_output_head_v1_listener head_listener = {
 	.name = head_handle_name,
 	.description = head_handle_description,
@@ -475,6 +484,7 @@ static const struct zwlr_output_head_v1_listener head_listener = {
 	.make = head_handle_make,
 	.model = head_handle_model,
 	.serial_number = head_handle_serial_number,
+	.adaptive_sync = head_handle_adaptive_sync,
 };
 
 static void output_manager_handle_head(void *data,
@@ -492,10 +502,14 @@ static void output_manager_handle_head(void *data,
 	zwlr_output_head_v1_add_listener(wlr_head, &head_listener, head);
 }
 
-static bool try_apply_profiles(struct kanshi_state *state) {
+static bool match_and_apply(struct kanshi_state *state) {
 	assert(wl_list_length(&state->heads) <= HEADS_MAX);
 	// matches[i] gives the kanshi_profile_output for the i-th head
 	struct kanshi_profile_output *matches[HEADS_MAX];
+	if (state->current_profile != NULL &&
+			match_profile(state, state->current_profile, matches)) {
+		return true; // keep the current profile if it still matches
+	}
 	struct kanshi_profile *profile = match(state, matches);
 	if (profile != NULL) {
 		apply_profile(state, profile, matches);
@@ -505,12 +519,21 @@ static bool try_apply_profiles(struct kanshi_state *state) {
 	return false;
 }
 
+bool kanshi_switch(struct kanshi_state *state, struct kanshi_profile *profile) {
+	struct kanshi_profile_output *matches[HEADS_MAX];
+	if (!match_profile(state, profile, matches)) {
+		return false;
+	}
+
+	apply_profile(state, profile, matches);
+	return true;
+}
+
 static void output_manager_handle_done(void *data,
 		struct zwlr_output_manager_v1 *manager, uint32_t serial) {
 	struct kanshi_state *state = data;
 	state->serial = serial;
-
-	try_apply_profiles(state);
+	match_and_apply(state);
 }
 
 static void output_manager_handle_finished(void *data,
@@ -530,8 +553,8 @@ static void registry_handle_global(void *data, struct wl_registry *registry,
 
 	if (strcmp(interface, zwlr_output_manager_v1_interface.name) == 0) {
 		uint32_t bind_version = 2;
-		if (version >= 3) {
-			bind_version = 3;
+		if (version >= 4) {
+			bind_version = 4;
 		}
 		state->output_manager = wl_registry_bind(registry, name,
 			&zwlr_output_manager_v1_interface, bind_version);
@@ -602,7 +625,7 @@ bool kanshi_reload_config(struct kanshi_state *state) {
 		state->config = config;
 		state->pending_profile = NULL;
 		state->current_profile = NULL;
-		return try_apply_profiles(state);
+		return match_and_apply(state);
 	}
 	return false;
 }
@@ -614,18 +637,31 @@ static const char usage[] = "Usage: %s [options...]\n"
 static const struct option long_options[] = {
 	{"help", no_argument, 0, 'h'},
 	{"config", required_argument, 0, 'c'},
+	{"listen-fd", required_argument, 0, 'l'},
 	{0},
 };
 
 int main(int argc, char *argv[]) {
 	const char *config_arg = NULL;
+#if KANSHI_HAS_VARLINK
+	int listen_fd = -1;
+#endif
 
 	int opt;
-	while ((opt = getopt_long(argc, argv, "hc:", long_options, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "hc:l:", long_options, NULL)) != -1) {
 		switch (opt) {
 		case 'c':
 			config_arg = optarg;
 			break;
+		case 'l':
+#if KANSHI_HAS_VARLINK
+			listen_fd = strtol(optarg, NULL, 10);
+#else
+			fprintf(stderr, "IPC support is disabled, "
+				"-l/--listen-fd is not supported\n");
+			return EXIT_FAILURE;
+#endif
+			break;
 		case 'h':
 			fprintf(stderr, usage, argv[0]);
 			return EXIT_SUCCESS;
@@ -654,7 +690,7 @@ int main(int argc, char *argv[]) {
 	};
 	int ret = EXIT_SUCCESS;
 #if KANSHI_HAS_VARLINK
-	if (kanshi_init_ipc(&state) != 0) {
+	if (kanshi_init_ipc(&state, listen_fd) != 0) {
 		ret = EXIT_FAILURE;
 		goto done;
 	}
@@ -663,7 +699,6 @@ int main(int argc, char *argv[]) {
 
 	struct wl_registry *registry = wl_display_get_registry(display);
 	wl_registry_add_listener(registry, &registry_listener, &state);
-	wl_display_dispatch(display);
 	wl_display_roundtrip(display);
 
 	if (state.output_manager == NULL) {
